<?php if (!defined('ABSPATH')) {exit;}
/**
 * Starts feed import
 *
 * @link       https://icopydoc.ru/
 * @since      1.0.0
 */

class IPYTW_Import_XML {
	private $pref = 'ipytw';
	protected $feed_id;

	public function __construct($feed_id) {
		$this->feed_id = $feed_id;
	}

	public function run() {
		new IPYTW_Error_Log('FEED № '.$this->get_feed_id().'; Стартовала run. Файл: class-ipytw-import-xml.php; Строка: '.__LINE__);
		$status_sborki = (int)ipytw_optionGET('ipytw_status_sborki', $this->get_feed_id());
		$time_start_last_step = (int)current_time('timestamp', 1); // unixtime

		$ipytw_last_element = (int)ipytw_optionGET('ipytw_last_element', $this->get_feed_id());
		$ipytw_step_import = ipytw_optionGET('ipytw_step_import', $this->get_feed_id(), 'set_arr');
		$timeout = $time_start_last_step + $ipytw_step_import - 2;

		$args_arr = array(
			'feed_id' => $this->get_feed_id(),
			'last_element' => $ipytw_last_element,
			'timeout' => $timeout
		);
		$import_obj = new IPYTW_Import_XML_Helper($args_arr); 

		switch ($status_sborki) {
			case 1: // загрузка файла с фидом на наш сервер
				new IPYTW_Error_Log('FEED № '.$this->get_feed_id().'; Первый шаг. Файл: class-ipytw-import-xml.php; Строка: '.__LINE__);
				do_action('ipytw_before_construct', $this->get_feed_id());

				$res_step = $this->copy_feed_from_source();
				if ($res_step === false) {
					$this->stop('copy_err');				
					return;
				} else {
					ipytw_optionUPD('ipytw_status_sborki', 2, $this->get_feed_id());
					return;
				}
			break;
			case 2: // создание категорий
				new IPYTW_Error_Log('FEED № '.$this->get_feed_id().'; Второй шаг. Файл: class-ipytw-import-xml.php; Строка: '.__LINE__);

				if ($import_obj->check_yml_format() === false) {
					new IPYTW_Error_Log('FEED № '.$this->get_feed_id().'; ERROR: Это не YML-формат Яндекса. Импорт невозможен. Файл: class-ipytw-import-xml.php; Строка: '.__LINE__);
					$this->stop('yml_format_err');
					ipytw_optionUPD('ipytw_err_list_code_arr', array('26'), $this->get_feed_id(), 'yes', 'set_arr');
					return;
				} else {
					ipytw_optionUPD('ipytw_err_list_code_arr', array(), $this->get_feed_id(), 'yes', 'set_arr');
				}
				
				$import_obj->import_cat();
				ipytw_optionUPD('ipytw_status_sborki', 3, $this->get_feed_id());

				// удалим файл со списком новых товаров т.к у нас начло сборки
				$name_dir = IPYTW_PLUGIN_UPLOADS_DIR_PATH;
				if (is_dir($name_dir)) {
					$filename1 = $name_dir.'/feed-importetd-product-ids-new-'.$this->get_feed_id().'.tmp';
					new IPYTW_Error_Log('FEED № '.$this->get_feed_id().'; $filename1 = '.$filename1.'. Файл: class-ipytw-import-xml.php; Строка: '.__LINE__);
					if (file_exists($filename1)) {unlink($filename1);}
				}
				return;
			break;
			case 3: // импорт товаров
				new IPYTW_Error_Log('FEED № '.$this->get_feed_id().'; Третий шаг. Файл: class-ipytw-import-xml.php; Строка: '.__LINE__);
		
				$ipytw_count_elements = (int)ipytw_optionGET('ipytw_count_elements', $this->get_feed_id(), 'set_arr');
				if ($ipytw_count_elements < 1) {
					if ($import_obj->set_count_elements() === true) {
						$ipytw_count_elements = (int)ipytw_optionGET('ipytw_count_elements', $this->get_feed_id(), 'set_arr');
						new IPYTW_Error_Log('FEED № '.$this->get_feed_id().'; После работы set_count_elements() $ipytw_count_elements = '.$ipytw_count_elements.'; Файл: class-ipytw-import-xml.php; Строка: '.__LINE__);
						ipytw_optionUPD('ipytw_last_element', 0, $this->get_feed_id()); // также устанавливаем значение счётчика обработанных товаров
					} else {
						new IPYTW_Error_Log('FEED № '.$this->get_feed_id().'; В фиде нет товаров! Останавливаем сборку; Файл: class-ipytw-import-xml.php; Строка: '.__LINE__);
						$this->stop('no_offers');
						return;
					}
				}
				$ipytw_last_element = (int)ipytw_optionGET('ipytw_last_element', $this->get_feed_id());

				$feed_imported_cat_arr = $this->get_imported_cat();
				new IPYTW_Error_Log('FEED № '.$this->get_feed_id().'; 2 $ipytw_last_element = '.$ipytw_last_element.'; $ipytw_count_elements = '.$ipytw_count_elements.'; Файл: class-ipytw-import-xml.php; Строка: '.__LINE__);
				if ($ipytw_last_element < $ipytw_count_elements-1) {
					new IPYTW_Error_Log('FEED № '.$this->get_feed_id().'; $ipytw_last_element = '.$ipytw_last_element.' < $ipytw_count_elements = '.$ipytw_count_elements.'; Файл: class-ipytw-import-xml.php; Строка: '.__LINE__);
					// ещё не все товары обработаны. Обработаем
					
					$xml_object = $import_obj->get_feed_our_site();
					$xml_offers_object = $xml_object->shop->offers;
					$ipytw_external = ipytw_optionGET('ipytw_external', $this->get_feed_id(), 'set_arr');

					for ($i = $ipytw_last_element; $i < $ipytw_count_elements; $i++) {
						$offer_xml_object = $xml_offers_object->offer[$i];

						$xml_offer_attr_object = $offer_xml_object->attributes(); // получим атрибуты тега offer
						if ($xml_offer_attr_object->group_id === null) { // это простой товар
							$feed_product_id = (string)$xml_offer_attr_object->id;	

							$parsing = new IPYTW_Import_Parsing($offer_xml_object, $feed_imported_cat_arr, $this->get_feed_id());
							$args_arr = $parsing->get_simple_arr();
							
							new IPYTW_Error_Log('FEED № '.$this->get_feed_id().'; $feed_product_id = '.$feed_product_id.'; $parsing->get_simple_arr() $args_arr = '.get_array_as_string($args_arr).';'.PHP_EOL.' Файл: class-ipytw-import-xml.php; Строка: '.__LINE__);
							new IPYTW_Error_Log('<<<'.serialize($args_arr).'>>>');

							if ($ipytw_external == 'enabled') { 
								$obj = new IPYTW_Import_Create_External_Product($args_arr, $offer_xml_object, $this->get_feed_id(), $feed_product_id);
							} else {
								$obj = new IPYTW_Import_Create_Simple_Product($args_arr, $offer_xml_object, $this->get_feed_id(), $feed_product_id);
							}
							
							$obj->add_product();
							$imported_ids = $obj->get_imported_ids();
						} else { // это вариативный товар
							$all_offers_of_variation_product = array();
							$all_offers_of_variation_product[] = $offer_xml_object;
							$feed_product_id = (string)$xml_offer_attr_object->group_id;
							new IPYTW_Error_Log('FEED № '.$this->get_feed_id().'; Работаем с вариативным товаром group_id = '.$feed_product_id.' ($feed_product_id); Файл: class-ipytw-import-xml.php; Строка: '.__LINE__);

							$group_id1 = (string)$xml_offer_attr_object->group_id;
							if (isset($xml_offers_object->offer[($i + 1)])) { // если существует следующий offer в фиде						
								for ($z = ($i + 1); $z < $ipytw_count_elements; $z++) {
									$offer_xml_object2 = $xml_offers_object->offer[$z];
									$xml_offer_attr_object2 = $offer_xml_object2->attributes();
									if ($xml_offer_attr_object2->group_id === null) {
										// это не вариативный товар.
										new IPYTW_Error_Log('Это простой товар '.$z);
										$z = $ipytw_count_elements;
									} else {
										$group_id2 = (string)$xml_offer_attr_object2->group_id;
										if ($group_id1 == $group_id2) {											
											$all_offers_of_variation_product[] = $offer_xml_object2;
											$i++; // эту вариацию смотреть не надо потом
										} else {
											$z = $ipytw_count_elements;
										}
									}
								}
							}
							
							if (count($all_offers_of_variation_product) == (int)1) {
								// одна вариация
								$ipytw_behaviour_one_variation = ipytw_optionGET('ipytw_behaviour_one_variation', $this->get_feed_id(), 'set_arr');
								if ($ipytw_behaviour_one_variation == 'add_as_simple') {
									$feed_product_id = (string)$xml_offer_attr_object->group_id; // передадим всё же group_id товара, а не id вариции на случай, если потом будет несколько вариантов в фиде
									new IPYTW_Error_Log('FEED № '.$this->get_feed_id().'; Пробуем импортировать вариативный с group_id = '.$feed_product_id.' как простой; Файл: class-ipytw-import-xml.php; Строка: '.__LINE__);
									
									$parsing = new IPYTW_Import_Parsing($offer_xml_object, $feed_imported_cat_arr, $this->get_feed_id());
									$args_arr = $parsing->get_simple_arr();
									if ($ipytw_external == 'enabled') { 
										$obj = new IPYTW_Import_Create_External_Product($args_arr, $offer_xml_object, $this->get_feed_id(), $feed_product_id);
									} else {
										$obj = new IPYTW_Import_Create_Simple_Product($args_arr, $offer_xml_object, $this->get_feed_id(), $feed_product_id);
									}
									$obj->add_product();
									$imported_ids = $obj->get_imported_ids();
								} else {
									new IPYTW_Error_Log('FEED № '.$this->get_feed_id().'; Пропускаем вариативный товар с единственной вариацией; Файл: class-ipytw-import-xml.php; Строка: '.__LINE__);
									$imported_ids = false; // пока пропустим
								}
							} else {
								$parsing = new IPYTW_Import_Parsing($all_offers_of_variation_product, $feed_imported_cat_arr, $this->get_feed_id());
								$args_arr = $parsing->get_variable_arr();
	
								new IPYTW_Error_Log('FEED № '.$this->get_feed_id().'; $feed_product_id = '.$feed_product_id.' ('.count($all_offers_of_variation_product).') вариации); $parsing->get_variable_arr() $args_arr = '.get_array_as_string($args_arr).';'.PHP_EOL.' Файл: class-ipytw-import-xml.php; Строка: '.__LINE__);
								new IPYTW_Error_Log('<<<'.serialize($args_arr).'>>>');
	
								$obj = new IPYTW_Import_Create_Variable_Product($args_arr, $all_offers_of_variation_product, $this->get_feed_id(), $feed_product_id);
								$obj->add_product();
								$imported_ids = $obj->get_imported_ids();
							}				
						}

						if ($imported_ids === false) {
							new IPYTW_Error_Log('FEED № '.$this->get_feed_id().'; $obj->get_imported_ids() вернула false; Файл: class-ipytw-import-xml.php; Строка: '.__LINE__);
						} else {
							new IPYTW_Error_Log('FEED № '.$this->get_feed_id().'; $obj->get_imported_ids() вернула $imported_ids; Файл: class-ipytw-import-xml.php; Строка: '.__LINE__);
							new IPYTW_Error_Log($imported_ids);
							$this->ipytw_wf($imported_ids); // записываем кэш-файл
						}
						
						// https://wp-kama.ru/handbook/codex/object-cache
						wp_suspend_cache_addition(true);
						$status_sborki = ipytw_optionGET('ipytw_status_sborki', $this->get_feed_id()); // ещё раз проверим необходимость остановки всей сборки
						wp_suspend_cache_addition(false);
						if ($status_sborki == -1) { // останавливаем сборку
							$this->stop('additional_stop');
							break;
						}
		
						ipytw_optionUPD('ipytw_last_element', $i, $this->get_feed_id());
		
						$unixtime = (int)current_time('timestamp', 1);
						if ($unixtime >= $timeout) {
							new IPYTW_Error_Log('FEED № '.$this->get_feed_id().'; $unixtime = '.$unixtime.' > $timeout = '.$timeout.'; Файл: class-ipytw-import-xml.php; Строка: '.__LINE__);
							break;
						} else {
							new IPYTW_Error_Log('FEED № '.$this->get_feed_id().'; $unixtime = '.$unixtime.' < $timeout = '.$timeout.'; Файл: class-ipytw-import-xml.php; Строка: '.__LINE__);
						}
					}
				} else {
					ipytw_optionUPD('ipytw_status_sborki', 4, $this->get_feed_id());
					return;					
				}
			break;
			case 4: 
				$ipytw_missing_product = ipytw_optionGET('ipytw_missing_product', $this->get_feed_id(), 'set_arr');
				if ($ipytw_missing_product === 'del' || $ipytw_missing_product === 'del_with_pic' || $ipytw_missing_product === 'out_of_stock') {
					$name_dir = IPYTW_PLUGIN_UPLOADS_DIR_PATH;
					if (is_dir($name_dir)) {
						$filename1 = $name_dir.'/feed-importetd-product-ids-new-'.$this->get_feed_id().'.tmp';
					}
				
					$name_dir = IPYTW_PLUGIN_UPLOADS_DIR_PATH;
					if (is_dir($name_dir)) {
						$args = array(
							'post_type' => 'product',   
							'post_status' => 'publish',
							'posts_per_page' => -1, 
							'relation' => 'AND',
							'fields' => 'ids',
							'meta_query' => array(
								array(
									'key' => '_ipytw_feed_id',
									'value' => (string)$this->get_feed_id()
								)
							)
						);	
						 
						$query = new WP_Query($args); 
						$old_ids_arr = array();
						if ($query->have_posts()) {
							$old_ids_arr = $query->posts; // id товаров, которые добавили в этой выгрузке
						}
				
						$filename1 = $name_dir.'/feed-importetd-product-ids-new-'.$this->get_feed_id().'.tmp';
						$string1 = file_get_contents($filename1);
						$res_arr = explode(PHP_EOL, $string1);
				
						$new_ids_arr = array(); // id товаров, которые добавили в этой выгрузке
						for ($i = 0; $i < count($res_arr); $i++) {
							$v = explode('-fisi-', $res_arr[$i]);
							$new_ids_arr[] = (int)$v[1]; 
						}
						
						$del_ids_arr = array();
						foreach($old_ids_arr as $id) {		 
							if (in_array($id, $new_ids_arr)) {

							} else { // нет в массиве
								$del_ids_arr[] = $id;
							}
						}
					
						if (!empty($del_ids_arr)) {
							$del_ids_arr = array_unique($del_ids_arr);
							foreach($del_ids_arr as $product_id) {
								if ($ipytw_missing_product === 'del' || $ipytw_missing_product === 'del_with_pic') {
									// !возможно нужно изменить последовательность удаления поста и вложения
									wooс_delete_product($product_id);

									if ($ipytw_missing_product === 'del_with_pic') {
										$remove_obj = new IPYTW_Remove_Attachments_Pictures($remove_obj);
										$remove_obj->remove_attachments($product_id);
									}
								}

								if ($ipytw_missing_product === 'out_of_stock') {
									$product = wc_get_product($product_id);
									$product->set_manage_stock(false);
									$product->set_stock_status('outofstock');
									// !возможно сюда надо дату обновления последнего синхрона
									$site_product_id = $product->save();
								}
							}
						}				
					}
				}
				$this->stop('full');
				return;
			break;
			default:
				new IPYTW_Error_Log('FEED № '.$this->get_feed_id().'; Шаг default; $status_sborki = '.$status_sborki.'. Файл: class-ipytw-import-xml.php; Строка: '.__LINE__);

				$this->stop('default');
				return;
		} // end switch($status_sborki)
		return; // final return from public function phase()
	} 

	public function copy_feed_from_source() {
		$ipytw_url_yml_file = ipytw_optionGET('ipytw_url_yml_file', $this->get_feed_id(), 'set_arr');
		if ($ipytw_url_yml_file == '') {
			new IPYTW_Error_Log('FEED № '.$this->get_feed_id().'; ERROR: Не указан URL фида: '.$ipytw_url_yml_file.'. Остановим сборку; Файл: offer.php; Строка: '.__LINE__); 
			return false;
		}
		$xml_string = ipytw_file_get_contents_curl($ipytw_url_yml_file, $this->get_feed_id());
		if ($xml_string === false) { 
			new IPYTW_Error_Log('ERROR: copy_feed: Нет доступа к файлу $ipytw_url_yml_file = '.$ipytw_url_yml_file.'; Файл: offer.php; Строка: '.__LINE__);
			return false;
		} else {	
			if (is_dir(IPYTW_PLUGIN_UPLOADS_DIR_PATH)) {
				$filename = IPYTW_PLUGIN_UPLOADS_DIR_PATH.'/'.$this->get_feed_id().'.xml';
				$fp = fopen($filename, "w");
				fwrite($fp, $xml_string); // записываем в файл текст
				fclose($fp); // закрываем
				ipytw_optionUPD('ipytw_copy_feed_from_source', $filename, $this->get_feed_id(), 'yes', 'set_arr');
				return true;
			} else {
				new IPYTW_Error_Log('ERROR: Нет папки '.IPYTW_PLUGIN_UPLOADS_DIR_PATH .'; Файл: functions.php; Строка: '.__LINE__);
				return false;
			}
		}	
	}

	public function ipytw_wf($imported_ids_arr) {
		$name_dir = IPYTW_PLUGIN_UPLOADS_DIR_PATH;
		if (!is_dir($name_dir)) {
			new IPYTW_Error_Log('WARNING: Папка $name_dir ='.$name_dir.' нет; Файл: functions.php; Строка: '.__LINE__);
			if (!mkdir($name_dir)) {
				new IPYTW_Error_Log('ERROR: Создать папку $name_dir ='.$name_dir.' не вышло; Файл: functions.php; Строка: '.__LINE__);
			} else { 
				// if (ipytw_optionGET('yzen_yandex_zen_rss') == 'enabled') {$result_yml = ipytw_optionGET('ipytw_feed_content');};
			}
		} else {
			// if (ipytw_optionGET('yzen_yandex_zen_rss') == 'enabled') {$result_yml = ipytw_optionGET('ipytw_feed_content');};
		}

		if (is_dir($name_dir)) {  
			$filename = $name_dir.'/feed-importetd-product-ids-new-'.$this->get_feed_id().'.tmp';
			$content = $imported_ids_arr['feed_product_id'].'-fisi-'.$imported_ids_arr['site_product_id'].PHP_EOL;
			file_put_contents($filename, $content, FILE_APPEND);
		} else {
			new IPYTW_Error_Log('ERROR: Нет папки import-from-yml! $name_dir ='.$name_dir.'; Файл: functions.php; Строка: '.__LINE__);
		}
	}

	/* для следующих версий
	public function write_log($imported_ids_arr) {
		$name_dir = IPYTW_PLUGIN_UPLOADS_DIR_PATH;
		if (!is_dir($name_dir)) {
			new IPYTW_Error_Log('WARNING: Папка $name_dir ='.$name_dir.' нет; Файл: functions.php; Строка: '.__LINE__);
			if (!mkdir($name_dir)) {
				new IPYTW_Error_Log('ERROR: Создать папку $name_dir ='.$name_dir.' не вышло; Файл: functions.php; Строка: '.__LINE__);
			} else { 
				// if (ipytw_optionGET('yzen_yandex_zen_rss') == 'enabled') {$result_yml = ipytw_optionGET('ipytw_feed_content');};
			}
		} else {
			// if (ipytw_optionGET('yzen_yandex_zen_rss') == 'enabled') {$result_yml = ipytw_optionGET('ipytw_feed_content');};
		}

		if (is_dir($name_dir)) {  
			$filename = $name_dir.'/feed-not-importetd-product-ids-'.$this->get_feed_id().'.tmp';
			$content = $imported_ids_arr['feed_product_id'].'-fisi-'.$imported_ids_arr['site_product_id'].PHP_EOL;
			file_put_contents($filename, $content, FILE_APPEND);
		} else {
			new IPYTW_Error_Log('ERROR: Нет папки import-from-yml! $name_dir ='.$name_dir.'; Файл: functions.php; Строка: '.__LINE__);
		}
	}*

	/*
	* С версии 1.0.0
	*
	* @return (array)
	* Возвращает массив просинхронизированных категорий. Где ключ - id массива в фиде. Значение - id категори у нас на сайте
	*/
	public function get_imported_cat() {
		new IPYTW_Error_Log('FEED № '.$this->get_feed_id().'; Стартовала get_imported_cat; Файл: functions.php; Строка: '.__LINE__);

		ipytw_check_dir(IPYTW_PLUGIN_UPLOADS_DIR_PATH);
		$filename = IPYTW_PLUGIN_UPLOADS_DIR_PATH.'/feed-importetd-cat-ids-'.$this->get_feed_id().'.tmp';

		if (is_file($filename)) {
			$result_arr = file_get_contents($filename);
			return unserialize($result_arr);
		} else { // Файла нет
			new IPYTW_Error_Log('FEED № '.$this->get_feed_id().'; ERROR: Нет файла: '.$filename.'; Файл: functions.php; Строка: '.__LINE__);
			return array();
		}
	} // end function get_imported_cat()

	public function stop($status = 'full') {
		$status_sborki = -1;
		ipytw_optionUPD('ipytw_status_sborki', $status_sborki, $this->get_feed_id());
		ipytw_optionUPD('ipytw_last_element', '-1', $this->get_feed_id());
		ipytw_optionUPD('ipytw_count_elements', '-1', $this->get_feed_id(), 'yes', 'set_arr');
		wp_clear_scheduled_hook('ipytw_cron_sborki', array($this->get_feed_id())); // останавливаем крон сборки
		do_action('ipytw_after_construct', $this->get_feed_id(), $status); // сборка закончена
	}

	protected function get_feed_id() {
		return $this->feed_id;
	}
}