<?php if (!defined('ABSPATH')) {exit;}
/**
 * The main class of the plugin Import from YML
 *
 * @package			Import from YML
 * @subpackage		
 * @since			0.1.0
 * 
 * @version			1.0.0
 * @author			Maxim Glazunov
 * @link			https://icopydoc.ru/
 * @see				
 * 
 * @param		
 *
 * @return		
 *
 * @depends			classes:	IPYTW_Data_Arr
 *								IPYTW_Settings_Page
 *								IPYTW_Debug_Page
 *								IPYTW_Error_Log
 *								IPYTW_Import_XML
 *					traits:	
 *					methods:	
 *					functions:	common_option_get
 *								common_option_upd
 *					constants:	IPYTW_PLUGIN_VERSION
 *								IPYTW_PLUGIN_BASENAME
 *								IPYTW_PLUGIN_DIR_URL
 *					options:	
 *
 */

final class ImportProductsYMLtoWooCommerce {
	private $site_uploads_url = IPYTW_SITE_UPLOADS_URL; // http://site.ru/wp-content/uploads
	private $site_uploads_dir_path = IPYTW_SITE_UPLOADS_DIR_PATH; // /home/site.ru/public_html/wp-content/uploads
	private $plugin_version = IPYTW_PLUGIN_VERSION; // 1.0.0
	private $plugin_upload_dir_url = IPYTW_PLUGIN_UPLOADS_DIR_URL; // http://site.ru/wp-content/uploads/import-from-yml/
	private $plugin_upload_dir_path = IPYTW_PLUGIN_UPLOADS_DIR_PATH; // /home/site.ru/public_html/wp-content/uploads/import-from-yml/
	private $plugin_dir_url = IPYTW_PLUGIN_DIR_URL; // http://site.ru/wp-content/plugins/import-from-yml/
	private $plugin_dir_path = IPYTW_PLUGIN_DIR_PATH; // /home/p135/www/site.ru/wp-content/plugins/import-from-yml/
	private $plugin_main_file_path = IPYTW_PLUGIN_MAIN_FILE_PATH; // /home/p135/www/site.ru/wp-content/plugins/import-from-yml/import-from-yml.php
	private $plugin_slug = IPYTW_PLUGIN_SLUG; // import-from-yml - псевдоним плагина
	private $plugin_basename = IPYTW_PLUGIN_BASENAME; // import-from-yml/import-from-yml.php - полный псевдоним плагина (папка плагина + имя главного файла)

	protected static $instance;
	public static function init() {
		if (is_null(self::$instance)) {
			self::$instance = new self();
		}
		return self::$instance;  
	}

	public static function on_activation() {
		if (!current_user_can('activate_plugins')) {return;}

		$name_dir = IPYTW_SITE_UPLOADS_DIR_PATH.'/import-from-yml';
		if (!is_dir($name_dir)) {
			if (!mkdir($name_dir)) {
				error_log('ERROR: Ошибка создания папки '.$name_dir.'; Файл: import-from-yml.php; Строка: '.__LINE__, 0);
			}
		}

		$ipytw_registered_feeds_arr = array(
			0 => array('last_id' => '1'),
			1 => array('id' => '1')
		);

		$def_plugin_date_arr = new IPYTW_Data_Arr();
		$ipytw_settings_arr = array();
		$ipytw_settings_arr['1'] = $def_plugin_date_arr->get_opts_name_and_def_date('all');

		if (is_multisite()) {
			add_blog_option(get_current_blog_id(), 'ipytw_keeplogs', '0');
			add_blog_option(get_current_blog_id(), 'ipytw_enable_backend_debug', '0');
			add_blog_option(get_current_blog_id(), 'ipytw_version', '1.9.5');
			add_blog_option(get_current_blog_id(), 'ipytw_disable_notices', '0');
			add_blog_option(get_current_blog_id(), 'ipytw_enable_five_min', '0');
			add_blog_option(get_current_blog_id(), 'ipytw_errors', '');
			add_blog_option(get_current_blog_id(), 'ipytw_settings_arr', $ipytw_settings_arr);
			add_blog_option(get_current_blog_id(), 'ipytw_status_sborki1', '-1');
			add_blog_option(get_current_blog_id(), 'ipytw_last_element1', '-1');
			add_blog_option(get_current_blog_id(), 'ipytw_registered_feeds_arr', $ipytw_registered_feeds_arr);
		} else {
			add_option('ipytw_keeplogs', '0');
			add_option('ipytw_enable_backend_debug', '0');
			add_option('ipytw_version', '1.9.5', '', 'no'); // без автозагрузки
			add_option('ipytw_disable_notices', '0', '', 'no');
			add_option('ipytw_enable_five_min', '0', '', 'no');
			add_option('ipytw_errors', '', '', 'no');
			add_option('ipytw_settings_arr', $ipytw_settings_arr);
			add_option('ipytw_status_sborki1', '-1');
			add_option('ipytw_last_element1', '-1');
			add_option('ipytw_registered_feeds_arr', $ipytw_registered_feeds_arr);
		}
	}
	
	public static function on_deactivation() {
		if (!current_user_can('activate_plugins')) {return;}

		$ipytw_registered_feeds_arr = common_option_get('ipytw_registered_feeds_arr', array());
		if (!empty($ipytw_registered_feeds_arr)) {
			for ($i = 1; $i < count($ipytw_registered_feeds_arr); $i++) { // с единицы, т.к инфа по конкретным фидам там
				$feed_id = $ipytw_registered_feeds_arr[$i]['id'];
				wp_clear_scheduled_hook('ipytw_cron_period', array($feed_id)); // отключаем крон
				wp_clear_scheduled_hook('ipytw_cron_sborki', array($feed_id)); // отключаем крон
			}
		}
	}

	public function __construct() {
		$this->check_options_upd(); // проверим, нужны ли обновления опций плагина
		$this->init_hooks(); // подключим хуки
 	}

	public function init_hooks() {
		add_action('admin_init', array($this, 'listen_submits_func'), 10);
		add_action('admin_init', array($this, 'admin_style_func'), 9999);
		add_action('admin_menu', array($this, 'add_admin_menu_func'), 10, 1);
		add_action('admin_notices', array($this, 'print_admin_notices_func'), 10, 1);
		add_action('woocommerce_product_options_general_product_data', array($this, 'add_to_product_sync_info_func'), 10, 1);

		add_action('ipytw_cron_sborki', array($this, 'ipytw_do_this_fifty_sec'), 10, 1);
		add_action('ipytw_cron_period', array($this, 'ipytw_do_this_event'), 10, 1);
		
		add_filter('plugin_action_links', array($this, 'plugin_action_links_func'), 10, 2);
		add_filter('upload_mimes', array($this, 'add_mime_types_func'), 99); // чутка позже остальных
		add_filter('cron_schedules', array($this, 'add_cron_intervals_func'), 10, 1);
	}

	public function check_options_upd() {
		if (false == common_option_get('ipytw_version')) { // это первая установка
			if (is_multisite()) {
				update_blog_option(get_current_blog_id(), 'ipytw_version', $this->plugin_version);
			} else {
				update_option('ipytw_version', $this->plugin_version);
			}
		} else {
			$this->set_new_options();
		}
	}

	public function get_plugin_version() {
		if (is_multisite()) {
			$v = get_blog_option(get_current_blog_id(), 'ipytw_version');
		} else {
			$v = get_option('ipytw_version');
		}
		return $v;
	}

	public function set_new_options() {
		// Если предыдущая версия плагина меньше текущей
		if (version_compare($this->get_plugin_version(), $this->plugin_version, '<')) { 

		} else { // обновления не требуются
			return;
		}

		/* Если версия плагина ниже 1.3.0 */
		$ipytw_version = ipytw_optionGET('ipytw_version');		
		if (version_compare($ipytw_version, '1.3.0', '<')) { 
			$ipytw_settings_arr = ipytw_optionGET('ipytw_settings_arr');
			$ipytw_settings_arr_keys_arr = array_keys($ipytw_settings_arr);
			for ($i = 0; $i<count($ipytw_settings_arr_keys_arr); $i++) {
				$key = $ipytw_settings_arr_keys_arr[$i];
				if (isset($ipytw_settings_arr[$key])) {
					$newkey = (string)$key; // преобразовываем ключ массива из числовго в строковый
					$ipytw_settings_arr_keys_arr[$newkey] = $ipytw_settings_arr_keys_arr[$i];
					unset($ipytw_settings_arr_keys_arr[$i]); // удаляем числовой ключ
					$feed_id = $key;
					wp_clear_scheduled_hook('ipytw_cron_period', array($feed_id)); // отключаем крон
					wp_clear_scheduled_hook('ipytw_cron_sborki', array($feed_id)); // отключаем крон
	
					$ipytw_status_sborki_key = 'ipytw_status_sborki'.$key;
					$ipytw_last_element_key = 'ipytw_last_element'.$key;
					ipytw_optionDEL($ipytw_status_sborki_key, $key);
					ipytw_optionDEL($ipytw_last_element_key, $key);
					ipytw_optionADD($ipytw_status_sborki_key, '-1');
					ipytw_optionADD($ipytw_last_element_key, '-1');
				}
			}
			ipytw_optionUPD('ipytw_settings_arr', $ipytw_settings_arr);
	
			$ipytw_registered_feeds_arr = ipytw_optionGET('ipytw_registered_feeds_arr');
			for ($i = 1; $i < count($ipytw_registered_feeds_arr); $i++) { // с единицы, т.к инфа по конкретным фидам там
				$feed_id = $ipytw_registered_feeds_arr[$i]['id'];
				$ipytw_registered_feeds_arr[$i]['id'] = (string)$feed_id;
			}	
			ipytw_optionUPD('ipytw_registered_feeds_arr', $ipytw_registered_feeds_arr);
		}
		/* end Если версия плагина ниже 1.3.0 */
	
		// получим список дефолтных настроек
		$ipytw_data_arr_obj = new IPYTW_Data_Arr();
		$opts_arr = $ipytw_data_arr_obj->get_opts_name_and_def_date_obj('all'); // список дефолтных настроек
		// проверим, заданы ли дефолтные настройки
		$ipytw_settings_arr = ipytw_optionGET('ipytw_settings_arr');
		$ipytw_settings_arr_keys_arr = array_keys($ipytw_settings_arr);
		for ($i = 0; $i < count($ipytw_settings_arr_keys_arr); $i++) {
			$feed_id = (string)$ipytw_settings_arr_keys_arr[$i];
			for ($n = 0; $n < count($opts_arr); $n++) {
				$name = $opts_arr[$n]->get_name();
				$value = $opts_arr[$n]->get_value();	
				if (!isset($ipytw_settings_arr[$feed_id][$name])) {ipytw_optionUPD($name, $value, $feed_id, 'yes', 'set_arr');}
			}
		}

		if (is_multisite()) {
			update_blog_option(get_current_blog_id(), 'ipytw_version', $this->plugin_version);
		} else {
			update_option('ipytw_version', $this->plugin_version);
		}
	}

	public function listen_submits_func() {
		do_action('ipytw_listen_submits');

		if (isset($_REQUEST['ipytw_submit_action'])) {
			$message = __('Updated', 'import-from-yml');
			$class = 'notice-success';
			if (isset($_POST['ipytw_run_cron']) && sanitize_text_field($_POST['ipytw_run_cron']) !== 'off') {
				$message .= '. '. __('Import products is running. You can continue working with the website', 'import-from-yml');
			}
	
			add_action('admin_notices', function() use ($message, $class) { 
				$this->admin_notices_func($message, $class);
			}, 10, 2);
		}
	}

	public function admin_style_func() {
		wp_register_style('ipytw-admin-css', IPYTW_PLUGIN_DIR_URL.'css/ipytw_style.css');
	}

	// Добавляем пункты меню
	public function add_admin_menu_func() {
		$page_suffix = add_menu_page(null , __('Import from YML', 'import-from-yml'), 'manage_options', 'ipytwexport', array($this, 'get_export_page_func'), 'dashicons-redo', 51);
		// создаём хук, чтобы стили выводились только на странице настроек
		add_action('admin_print_styles-'. $page_suffix, array($this, 'admin_css_func'));

		$page_suffix = add_submenu_page('ipytwexport', __('Debug', 'import-from-yml'), __('Debug page', 'import-from-yml'), 'manage_woocommerce', 'ipytwdebug', array($this, 'get_debug_page_func'));
		add_action('admin_print_styles-'. $page_suffix, array($this, 'admin_css_func'));

		$page_subsuffix = add_submenu_page('ipytwexport', __('Add Extensions', 'import-from-yml'), __('Extensions', 'import-from-yml'), 'manage_options', 'ipytwextensions', 'ipytw_extensions_page');
		require_once IPYTW_PLUGIN_DIR_PATH.'/extensions.php';
		add_action('admin_print_styles-'. $page_subsuffix, array($this, 'admin_css_func'));
	} 

	// вывод страницы настроек плагина
	public function get_export_page_func() {
		new IPYTW_Settings_Page();
		return;
	} 

	// вывод страницы отладки плагина
	public function get_debug_page_func() {
		new IPYTW_Debug_Page();
		return;
	} 

	// Вывод различных notices
	public function print_admin_notices_func() {
		$ipytw_enable_backend_debug = ipytw_optionGET('ipytw_enable_backend_debug'); 
		if ($ipytw_enable_backend_debug === 'on') {
			print '<div class="updated notice notice-success is-dismissible"><p>';
			print '<h1>Зарегистрированные фиды:</h1>';
			$ipytw_registered_feeds_arr = ipytw_optionGET('ipytw_registered_feeds_arr');
			echo get_array_as_string($ipytw_registered_feeds_arr, '<br/>');

			print '<h1>Статусы сборок:</h1>';
			for ($i = 1; $i < count($ipytw_registered_feeds_arr); $i++) { // с единицы, т.к инфа по конкретным фидам там
				$feed_id = $ipytw_registered_feeds_arr[$i]['id'];
				$ipytw_status_sborki_val = ipytw_optionGET('ipytw_status_sborki', $feed_id);
				$ipytw_last_element_val = ipytw_optionGET('ipytw_last_element', $feed_id);
				if ($feed_id === '0' || $feed_id === 0) {$feed_id = '';}
				$ipytw_status_sborki_key = 'ipytw_status_sborki'.$feed_id;
				$ipytw_last_element_key = 'ipytw_last_element'.$feed_id;
				print '$ipytw_status_sborki_key = '.$ipytw_status_sborki_key.'; $ipytw_status_sborki_val = '.$ipytw_status_sborki_val.'<br/>'; 
				print '$ipytw_last_element_key = '.$ipytw_last_element_key.'; $ipytw_last_element_val = '.$ipytw_last_element_val.'<br/>';
			}

			print '<h1>Динамические опции:</h1>';
			$ipytw_settings_arr = ipytw_optionGET('ipytw_settings_arr');
			echo get_array_as_string($ipytw_settings_arr, '<br/>');

			print '<h1>Крон задачи (ipytw_cron_period) и (ipytw_cron_sborki):</h1>';
			$cron_zadachi = get_option('cron'); // получаем все задачи из базы данных
			echo get_array_as_string($cron_zadachi, '<br/>'); // можно использовать функции print_r() или var_dump() для вывода всех задач
			print '</p></div>';
		}

		$ipytw_disable_notices = ipytw_optionGET('ipytw_disable_notices');
		if ($ipytw_disable_notices === 'on') {} else {
			$ipytw_registered_feeds_arr = ipytw_optionGET('ipytw_registered_feeds_arr');
			for ($i = 1; $i < count($ipytw_registered_feeds_arr); $i++) { // с единицы, т.к инфа по конкретным фидам там
				$feed_id = $ipytw_registered_feeds_arr[$i]['id'];		
				$status_sborki = ipytw_optionGET('ipytw_status_sborki', $feed_id); // ожидается тип (int)
				switch ($status_sborki) {
					case 1:
						print '<div class="updated notice notice-success is-dismissible"><p>'. __('Feed ID', 'import-from-yml').' = '.$feed_id.'. '. __('Import products is running', 'import-from-yml').'. '. __('Step', 'import-from-yml').': 1. '. __('Getting a feed from the source site', 'import-from-yml').'.</p></div>';
					break;
					case 2:
						print '<div class="updated notice notice-success is-dismissible"><p>'. __('Feed ID', 'import-from-yml').' = '.$feed_id.'. '. __('Import products is running', 'import-from-yml').'. '. __('Step', 'import-from-yml').': 2. '. __('Importing a list of categories', 'import-from-yml').'.</p></div>';
					break;
					case 3:
						$ipytw_last_element = (int)ipytw_optionGET('ipytw_last_element', $feed_id) + 1;
						$ipytw_count_elements = ipytw_optionGET('ipytw_count_elements', $feed_id, 'set_arr');
						if ($ipytw_count_elements == -1) {$ipytw_count_elements = '('. __('products are being counted', 'import-from-yml').'...)';}
						print '<div class="updated notice notice-success is-dismissible"><p>'. __('Feed ID', 'import-from-yml').' = '.$feed_id.'. '. __('Import products is running', 'import-from-yml').'. '. __('Step', 'import-from-yml').': 3. '. __('Processed products', 'import-from-yml').': '.$ipytw_last_element.' '. __('of', 'import-from-yml').' '.$ipytw_count_elements.'.</p></div>';
					break;
					case 4:
						print '<div class="updated notice notice-success is-dismissible"><p>'. __('Removing products', 'import-from-yml').'...</p></div>';
					break;
					default:
				}
			}
		}
	}

	public static function admin_css_func() { // Ставим css-файл в очередь на вывод	
		wp_enqueue_style('ipytw-admin-css'); 
	}

	public function add_to_product_sync_info_func() { 
		global $product, $post;

		if (get_post_meta($post->ID, '_ipytw_feed_product_id', true) !== '') {
			$feed_product_id = get_post_meta($post->ID, '_ipytw_feed_product_id', true);
		}

		if (get_post_meta($post->ID, '_ipytw_feed_id', true) !== '') {
			$feed_id = get_post_meta($post->ID, '_ipytw_feed_id', true);
		}

		if (get_post_meta($post->ID, '_ipytw_date_last_import', true) !== '') {
			$date_last_import = get_post_meta($post->ID, '_ipytw_date_last_import', true);
		}

		if (!empty($feed_product_id) && !empty($feed_id) && !empty($feed_product_id)) {
			printf('<div class="hide-if-no-js"><p>%1$s: "%2$s" (%3$s). %4$s: "%5$s"</p></div>', 
				__('The product was imported from the feed ID', 'import-from-yml'),
				$feed_id,
				$date_last_import,
				__('His ID in this feed', 'import-from-yml'),
				$feed_product_id		
			);
		}
	}

	/* Фильтры */

	public function plugin_action_links_func($actions, $plugin_file) {
		if (false === strpos($plugin_file, IPYTW_PLUGIN_BASENAME)) { // проверка, что у нас текущий плагин
			return $actions;
		}
		$settings_link = '<a style="color: green; font-weight: 700;" href="/wp-admin/admin.php?page=ipytwextensions">'. __('More features', 'import-from-yml').'</a>';
		array_unshift($actions, $settings_link); 	
		$settings_link = '<a href="/wp-admin/admin.php?page=ipytwexport">'. __('Settings', 'import-from-yml').'</a>';
		array_unshift($actions, $settings_link); 
		$actions = apply_filters('ipytw_plugin_action_links_after', $actions, $settings_link, $plugin_file);
		return $actions; 
	}

	// Разрешим загрузку xml и csv файлов
	public function add_mime_types_func($mimes) {
		$mimes ['csv'] = 'text/csv';
		$mimes ['xml'] = 'text/xml';
		$mimes ['yml'] = 'text/xml';
		return $mimes;
	} 

	// добавляем интервалы крон в 70 секунд и 6 часов 
	public function add_cron_intervals_func($schedules) {
		$schedules['fifty_sec'] = array(
			'interval' => 61,
			'display' => __('61 seconds', 'import-from-yml')
		);
		$schedules['five_min'] = array(
			'interval' => 300,
			'display' => __('5 minutes', 'import-from-yml')
		);
		$schedules['six_hours'] = array(
			'interval' => 21600,
			'display' => __('6 hours', 'import-from-yml')
		);
		$schedules['every_two_days'] = array(
			'interval' => 172800,
			'display' => __('Every two days', 'import-from-yml')
		);
		$schedules['week'] = array(
			'interval' => 604800,
			'display' => __('1 week', 'import-from-yml')
		);
		return $schedules;
	}

	/* функции крона */
	public function ipytw_do_this_event($feed_id) {
		new IPYTW_Error_Log('FEED № '.$feed_id.'; Крон ipytw_do_this_event включен. Делаем что-то каждый час; Файл: import-from-yml.php; Строка: '.__LINE__);

		ipytw_optionUPD('ipytw_status_sborki', '1', $feed_id);
		ipytw_optionUPD('ipytw_last_element', '0', $feed_id); // выставляем в положения начала сборки

		wp_clear_scheduled_hook('ipytw_cron_sborki', array($feed_id));
		// Возвращает nul/false. null когда планирование завершено. false в случае неудачи.
		$res = wp_schedule_event(time(), 'fifty_sec', 'ipytw_cron_sborki', array($feed_id));
		if ($res === false) {
			new IPYTW_Error_Log('FEED № '.$feed_id.'; ERROR: Не удалось запланировань CRON fifty_sec; Файл: import-from-yml.php; Строка: '.__LINE__);
		} else {
			new IPYTW_Error_Log('FEED № '.$feed_id.'; CRON fifty_sec успешно запланирован; Файл: import-from-yml.php; Строка: '.__LINE__);
		}
	}
	public function ipytw_do_this_fifty_sec($feed_id) {
		new IPYTW_Error_Log('Cтартовала крон-задача do_this_seventy_sec');
		$import = new IPYTW_Import_XML($feed_id); // делаем что-либо каждые 50 сек
		$import->run();
	}
 	/* end функции крона */

	private function admin_notices_func($message, $class) {
		$ipytw_disable_notices = ipytw_optionGET('ipytw_disable_notices');
		if ($ipytw_disable_notices === 'on') {
			return;
		} else {
			printf('<div class="notice %1$s"><p>%2$s</p></div>', $class, $message);
			return;
		}
	}
} /* end class ImportProductsYMLtoWooCommerce */